#include <LiquidCrystal.h>
#include <SoftwareSerial.h>
#include <TimeLib.h>
#include <TinyGPS++.h>
#include <Sunrise.h>

// https://github.com/PaulStoffregen/Time

TinyGPSPlus gps;

byte c_minute;
byte c_hour;
byte c_second;

byte s_minute = 0;
byte s_hour = 0 ;
byte s_second = 0;

byte r_minute = 0;
byte r_hour = 0;
byte r_second = 0;

boolean tochki_stat = false;

unsigned long currentMillis;
unsigned long ClockInterval = 0;
unsigned long gpsInterval = 0;

// DMC24227 LCD

const int MSK = 3;

const int rs = 10,
          en = 9,
          d4 = 8,
          d5 = 5,
          d6 = 7,
          d7 = 6;

const int digitWidth = 3;
const char bigDigitsTop[10][digitWidth] = { 3, 0, 3, 0, 3, 32, 2, 2, 3, 0, 2, 3, 3, 1, 3, 3, 2, 2, 3, 2, 2, 0, 0, 3, 3, 2, 3, 3, 2, 3 };
const char bigDigitsBot[10][digitWidth] = { 3, 1, 3, 1, 3, 1, 3, 1, 1, 1, 1, 3, 32, 32, 3, 1, 1, 3, 3, 1, 3, 32, 32, 3, 3, 1, 3, 1, 1, 3 };

LiquidCrystal lcd(rs, en, d4, d5, d6, d7);

SoftwareSerial ss(2, 11); // RX,TX GPS

byte glyphs[5][8] = {
  {B11111, B11111, B00000, B00000, B00000, B00000, B00000, B00000},
  {B00000, B00000, B00000, B00000, B00000, B00000, B11111, B11111},
  {B11111, B11111, B00000, B00000, B00000, B00000, B11111, B11111},
  {B11111, B11111, B11111, B11111, B11111, B11111, B11111, B11111},
  {B00000, B00000, B00000, B00000, B00000, B01110, B01110, B01110}
};

boolean gps_status = false; // First Start GPS Clock


void setup() {

  ss.begin(9600);

  pinMode(13, OUTPUT);   // Buzzer and Led
  digitalWrite(13, LOW); // Turn Off

  lcd.begin(16, 2);

  for (int i = 0; i < 5; i++) {
    lcd.createChar(i, glyphs[i]);
  }

  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(F("GPS Clock Verison 1.0"));
  lcd.setCursor(0, 1);
  lcd.print(F("26.09.2018 Roma Kuzmin"));
  delay(3000);
  lcd.clear();

}

// ---------------------- Main Loop --------------------------

void loop() {

  while (ss.available() > 0) {
    gps.encode(ss.read());
  }

  currentMillis = millis();

  if (currentMillis - ClockInterval > 1000 ) {  // Display Clock
    ClockInterval = currentMillis;

    c_hour = hour();
    c_minute = minute();
    c_second = second();

    if (c_hour < 10) {
      showNumber(0, 0);
      showNumber(c_hour, 1);
    } else showNumber(c_hour, 0);

    if (c_minute < 10) {
      showNumber(0, 2);
      showNumber(c_minute, 3);
    } else showNumber(c_minute, 2);

    if (gps.location.isValid() && gps.date.isValid() && gps.time.isValid()) {

      if (!gps_status) {
        get_time();
        get_sun();
        gps_status = true;
      }

      lcd.setCursor(16, 1);
      if (r_hour < 10) lcd.print(F("0")); lcd.print(r_hour); lcd.print(":");
      if (r_minute < 10) lcd.print(F("0")); lcd.print(r_minute); lcd.print(":");
      if (r_second < 10) lcd.print(F("0")); lcd.print(r_second);

      lcd.setCursor(16, 0);
      if (s_hour < 10) lcd.print(F("0")); lcd.print(s_hour); lcd.print(":");
      if (s_minute < 10) lcd.print(F("0")); lcd.print(s_minute); lcd.print(":");
      if (s_second < 10) lcd.print(F("0")); lcd.print(s_second);

    } else {
      lcd.setCursor(16, 0);
      lcd.print(F("Looking."));
      lcd.setCursor(16, 1);
      lcd.print(F("GPS Sat."));
      gps_status = false;
    }

    tochki(tochki_stat);

    tochki_stat = !tochki_stat;

  }

  if (currentMillis - gpsInterval > (60000 * 5) ) { // 5 minute
    gpsInterval = currentMillis;
    if (gps_status) {
      get_time();
      get_sun();
    }
  }

}

// --------------------------------------------- Functions ----------------------------------------------------

void get_time( void ) {

  if (gps.location.isValid() && gps.date.isValid() && gps.time.isValid()) {

    setTime(gps.time.hour(),
            gps.time.minute(),
            gps.time.second(),
            gps.date.day(),
            gps.date.month(),
            gps.date.year());
    adjustTime(MSK * SECS_PER_HOUR);
  }
}

void get_sun( void ) {

  int t;

  if (gps.location.isValid() && gps.date.isValid() && gps.time.isValid()) {

    Sunrise mySunrise(gps.location.lat(), gps.location.lng(), MSK);

    mySunrise.Actual();

    t = mySunrise.Rise(gps.date.month(), gps.date.day()); // Month,Day

    if (t >= 0) {
      r_hour = mySunrise.Hour();
      r_minute = mySunrise.Minute();
    } else {
      r_hour = 0;
      r_minute = 0;
    }

    t = mySunrise.Set(gps.date.month(), gps.date.day()); // Month,Day

    if (t >= 0) {
      s_hour = mySunrise.Hour();
      s_minute = mySunrise.Minute();
    } else {
      s_hour = 0;
      s_minute = 0;
    }

  }
}

void showDigit(int digit, int position) {

  lcd.setCursor(position * (digitWidth + 1), 0);

  for (int i = 0; i < digitWidth; i++) lcd.print(bigDigitsTop[digit][i]);

  lcd.setCursor(position * (digitWidth + 1), 1);

  for (int i = 0; i < digitWidth; i++) lcd.print(bigDigitsBot[digit][i]);

}

void showNumber(int value, int position) {

  int index;

  char buffer[12];

  itoa(value, buffer, 10);

  for (index = 0; index < 10; index++) {
    char c = buffer[index];
    if ( c == 0) return;
    c = c - 48;
    showDigit(c, position + index);
  }

}

void tochki( boolean stat ) {

  if (stat) {
    lcd.setCursor(7, 0);
    lcd.print(F("*"));
    lcd.setCursor(7, 1);
    lcd.print(F("*"));
    // digitalWrite(13, HIGH);
  } else {
    lcd.setCursor(7, 0);
    lcd.print(F(" "));
    lcd.setCursor(7, 1);
    lcd.print(F(" "));
    // digitalWrite(13, LOW);
  }

}
